==> nodejs_interop-0.7.0.tar.gz: lib/src/node/process/process.dart <==
part of nodejs_interop.node;

/// The CPU time usage of a process.
@JS()
abstract class CpuUsage {

  /// Time spent in system code, in microseconds.
  external int get system;

  /// Time spent in user code, in microseconds.
  external int get user;
}

/// Provides options for emitting a process warning.
@JS()
@anonymous
class EmitWarningOptions {

  /// Creates a new options object.
  external factory EmitWarningOptions({String code, ctor, String detail, String type = 'Warning'});

  /// A unique identifier for the warning instance being emitted.
  external String get code;
  external set code(String value);

  /// An optional function used to limit the generated stack trace. Defaults to [NodeProcess.emitWarning].
  external dynamic get ctor;
  external set ctor(value);

  /// Additional text to include with the error.
  external String get detail;
  external set detail(String value);

  /// The name to use for the type of warning being emitted.
  external String get type;
  external set type(String value);
}

/// The memory usage of a process.
@JS()
abstract class MemoryUsage {

  /// The memory usage of C++ objects bound to JavaScript objects managed by V8, in bytes.
  external int get external;

  /// The total amount of space occupied by V8's heap, in bytes.
  external int get heapTotal;

  /// The current amount of space used by V8's heap, in bytes.
  external int get heapUsed;

  /// The amount of space occupied in the main memory device, in bytes.
  external int get rss;
}

/// Provides information about, and control over, the current [Node.js](https://nodejs.org) process.
@JS()
abstract class NodeProcess extends EventEmitter {

  /// The processor architecture that the process is currently running on.
  external String get arch;

  /// The command line arguments passed when the process was launched.
  ///
  /// The first element will be [execPath]. See [argv0] if access to the original value of `argv[0]` is needed.
  /// The second element will be the path to the JavaScript file being executed.
  /// The remaining elements will be any additional command line arguments.
  external List<String> get argv;

  /// The original value of `argv[0]` passed when Node.js starts.
  external String get argv0;

  /// If the process was spawned with an IPC channel, the [channel] property is a reference to the IPC channel.
  /// If no IPC channel exists, this property is `null`.
  external dynamic get channel;

  /// An object containing the JavaScript representation of the configure options used to compile the current Node.js executable.
  /// This is the same as the `config.gypi` file that was produced when running the `./configure` script.
  external dynamic get config;
  external set config(value);

  /// If the process is spawned with an IPC channel, the [connected] property will return `true` so long as the IPC channel is connected.
  /// It will return `false` after [disconnect] is called.
  external bool get connected;

  /// An object containing the user environment.
  external dynamic get env;

  /// The list of Node.js-specific command-line options passed when the process was launched.
  ///
  /// These options do not appear in the array returned by the [argv] property, and do not include the Node.js executable,
  /// the name of the script, or any options following the script name.
  /// These options are useful in order to spawn child processes with the same execution environment as the parent.
  external List<String> get execArgv;

  /// The absolute pathname of the executable that started the process.
  external String get execPath;

  /// The process exit code, when the process either exits gracefully, or is exited via [exit] without specifying a code.
  external int get exitCode;
  external set exitCode(int value);

  /// An alternative way of retrieving `require.main`.
  external Module get mainModule;

  /// The process identifier (PID) of the process.
  external int get pid;

  /// A string identifying the operating system platform on which the process is running.
  external String get platform;

  /// The PID of the current parent process.
  external int get ppid;

  /// The metadata related to the current release, including URLs for the source tarball and headers-only tarball.
  external NodeRelease get release;

  /// A stream connected to `stderr` (file descriptor `2`).
  /// It is a `Socket` (which is a [Duplex] stream) unless file descriptor `2` refers to a file, in which case it is a [Writable] stream.
  external Writable get stderr;

  /// A stream connected to `stdin` (file descriptor `0`).
  /// It is a `Socket` (which is a [Duplex] stream) unless file descriptor `0` refers to a file, in which case it is a [Readable] stream.
  external Readable get stdin;

  /// A stream connected to `stdout` (file descriptor `1`).
  /// It is a `Socket` (which is a [Duplex] stream) unless file descriptor `1` refers to a file, in which case it is a [Writable] stream.
  external Writable get stdout;

  /// The current process title (i.e. the value of the `ps` command).
  external String get title;
  external set title(String value);

  /// The Node.js version string.
  external String get version;

  /// An object listing the version strings of Node.js and its dependencies.
  external dynamic get versions;

  /// Causes the process to exit immediately and generate a core file.
  external void abort();

  /// Changes the current working [directory] of the process.
  external void chdir(String directory);

  /// Returns the user and system CPU time usage of the current process.
  /// The result of a previous call to [cpuUsage] can be passed as the argument to the function, to get a diff reading.
  external CpuUsage cpuUsage([CpuUsage previousValue]);

  /// Returns the current working directory of the process.
  external String cwd();

  /// If the process is spawned with an IPC channel, the method will close the IPC channel to the parent process,
  /// allowing the child process to exit gracefully once there are no other connections keeping it alive.
  ///
  /// The effect of calling [disconnect] is that same as calling the parent process's `ChildProcess.disconnect()`.
  external void disconnect();

  /// Allows to dynamically load shared objects.
  ///
  /// It is primarily used by [require] to load C++ Addons, and should not be used directly, except in special cases.
  /// In other words, [require] should be preferred over [dlopen], unless there are specific reasons.
  ///
  /// The [flags] argument allows to specify [dlopen] behavior. See the `OSDlopenConstants` documentation for details.
  ///
  /// If there are specific reasons to use [dlopen], it's often useful to use `require.resolve()` to look up the module's path.
  ///
  /// Note: An important drawback when calling [dlopen] is that the `module` instance must be passed.
  /// Functions exported by the C++ Addon will be accessible via `module.exports`.
  external void dlopen(Module module, String filename, [int flags]);

  /// Emits custom or application specific process warnings.
  /// These can be listened for by adding a handler to the `process.on("warning")` event.
  ///
  /// If [warning] is passed as a [JSError] object, the optional arguments are ignored.
  external void emitWarning(warning, [typeOrOptions, codeOrCtor, Function ctor]);

  /// Instructs Node.js to terminate the process synchronously with an exit status of [code].
  /// If [code] is omitted, [exit] uses either the success code `0` or the value of [exitCode] if it has been set.
  void exit([int code]);

  /// Returns the numerical effective group identity of process (see [`getegid()`](http://manpages.ubuntu.com/manpages/xenial/man2/getegid.2.html)).
  external int getegid();

  /// Returns the numerical effective user identity of the process (see [`getegid()`](http://manpages.ubuntu.com/manpages/xenial/man2/geteuid.2.html)).
  external int geteuid();

  /// Returns the numeric group identity of the process (see [`getegid()`](http://manpages.ubuntu.com/manpages/xenial/man2/getgid.2.html)).
  external int getgid();

  /// Returns a list with the supplementary group identifiers (see [`getegid()`](http://manpages.ubuntu.com/manpages/xenial/man2/getgroups.2.html)).
  external List<int> getgroups();

  /// Returns the numeric user identity of the process (see [`getegid()`](http://manpages.ubuntu.com/manpages/xenial/man2/getuid.2.html)).
  external int getuid();

  /// Returns a value indicating whether a callback has been set using [setUncaughtExceptionCaptureCallback].
  external bool hasUncaughtExceptionCaptureCallback();

  /// Returns the current high-resolution real time in a `[seconds, nanoseconds]` tuple,
  /// where `nanoseconds` is the remaining part of the real time that can't be represented in second precision.
  ///
  /// [time] is an optional parameter that must be the result of a previous [hrtime] call to diff with the current time.
  external List<int> hrtime([List<int> time]);

  /// Reads the `/etc/group` file and initializes the group access list, using all groups of which the [user] is a member.
  /// This is a privileged operation that requires that the process either have `root` access or the `CAP_SETGID` capability.
  ///
  /// [user] can be a user name or a numeric user identifier.
  /// [extraGroup] can be a group name or a numeric group identifier.
  external void initgroups(user, [extraGroup]);

  /// Sends the [signal] to the process identified by [pid].
  ///
  /// Even though the name of this method is `process.kill()`, it is really just a signal sender,
  /// like the [`kill`](http://manpages.ubuntu.com/manpages/xenial/man2/kill.2.html) system call.
  /// The signal sent may do something other than kill the target process.
  external void kill(int pid, [signal = 'SIGTERM']);

  /// Returns an object describing the memory usage of the process measured in bytes.
  external MemoryUsage memoryUsage();

  /// Adds the [callback] to the "next tick queue".
  /// Once the current turn of the event loop turn runs to completion, all callbacks currently in the next tick queue will be called.
  external void nextTick(Function callback, [arg1, arg2, arg3, arg4, arg5, arg6, arg7]);

  /// If Node.js is spawned with an IPC channel, the [send] method can be used to send messages to the parent process.
  /// Messages will be received as a `message` event on the parent's `ChildProcess` object.
  ///
  /// If Node.js was not spawned with an IPC channel, the [send] method will be not be available.
  ///
  /// The message goes through JSON serialization and parsing. The resulting message might not be the same as what is originally sent.
  external bool send(message, [sendHandleOrOptions, optionsOrCallback, CompletionCallback0 callback]);

  /// Sets the effective group identity of the process (see [`setegid()`](http://manpages.ubuntu.com/manpages/xenial/man2/setegid.2.html)).
  ///
  /// The [id] can be passed as either a numeric identifier or a group name string.
  /// If a group name is specified, the method blocks while resolving the associated numeric identifier.
  external void setegid(id);

  /// Sets the effective user identity of the process (see [`seteuid()`](http://manpages.ubuntu.com/manpages/xenial/man2/seteuid.2.html)).
  ///
  /// The [id] can be passed as either a numeric identifier or a username string.
  /// If a username is specified, the method blocks while resolving the associated numeric identifier.
  external void seteuid(id);

  /// Sets the group identity of the process (see [`setgid()`](http://manpages.ubuntu.com/manpages/xenial/man2/setgid.2.html)).
  ///
  /// The [id] can be passed as either a numeric identifier or a group name string.
  /// If a group name is specified, the method blocks while resolving the associated numeric identifier.
  external void setgid(id);

  /// Sets the supplementary group identifiers for the process (see [`setgroups()`](http://manpages.ubuntu.com/manpages/xenial/man2/setgroups.2.html)).
  /// This is a privileged operation that requires the process to have `root` or the `CAP_SETGID` capability.
  ///
  /// The [groups] list can contain numeric group identifiers, group names or both.
  external void setgroups(List groups);

  /// Sets the user identity of the process (see [`setuid()`](http://manpages.ubuntu.com/manpages/xenial/man2/setuid.2.html)).
  ///
  /// The [id] can be passed as either a numeric identifier or a username string.
  /// If a username is specified, the method blocks while resolving the associated numeric identifier.
  external void setuid(id);

  /// Sets a function that will be invoked when an uncaught exception occurs, which will receive the exception value itself as its first argument.
  external void setUncaughtExceptionCaptureCallback(void fn(JSError error));

  /// Sets or returns the process's file mode creation mask.
  ///
  /// Invoked without an argument, the current mask is returned,
  /// otherwise the `umask` is set to the argument value and the previous mask is returned.
  external int umask([int mask]);

  /// Returns the number of seconds the current process has been running.
  /// The returned value includes fractions of a second. Use [double.floor] to get whole seconds.
  external double uptime();
}

/// Provides information about a [Node.js](https://nodejs.org) release.
@JS()
abstract class NodeRelease {

  /// An absolute URL pointing to a `.tar.gz` file containing the source code of this release.
  external String get headersUrl;

  /// An absolute URL pointing to a `node.lib` file matching the architecture and version of this release.
  external String get libUrl;

  /// A string label identifying the [LTS](https://github.com/nodejs/Release) label for this release.
  /// This property only exists for LTS releases.
  external String get lts;

  /// A value that will always be `"node"` for Node.js.
  external String get name;

  /// An absolute URL pointing to a `.tar.gz` file containing only the source header files for this release.
  external String get sourceUrl;
}

==> nodejs_interop-0.8.0.tar.gz: lib/src/node/process/process.dart <==
part of nodejs_interop.node;

/// The CPU time usage of a process.
@JS()
abstract class CpuUsage {

  /// Time spent in system code, in microseconds.
  external int get system;

  /// Time spent in user code, in microseconds.
  external int get user;
}

/// Provides options for emitting a process warning.
@JS()
@anonymous
class EmitWarningOptions {

  /// Creates a new options object.
  external factory EmitWarningOptions({String code, ctor, String detail, String type = 'Warning'});

  /// A unique identifier for the warning instance being emitted.
  external String get code;
  external set code(String value);

  /// An optional function used to limit the generated stack trace. Defaults to [Process.emitWarning].
  external dynamic get ctor;
  external set ctor(value);

  /// Additional text to include with the error.
  external String get detail;
  external set detail(String value);

  /// The name to use for the type of warning being emitted.
  external String get type;
  external set type(String value);
}

/// The memory usage of a process.
@JS()
abstract class MemoryUsage {

  /// The memory usage of C++ objects bound to JavaScript objects managed by V8, in bytes.
  external int get external;

  /// The total amount of space occupied by V8's heap, in bytes.
  external int get heapTotal;

  /// The current amount of space used by V8's heap, in bytes.
  external int get heapUsed;

  /// The amount of space occupied in the main memory device, in bytes.
  external int get rss;
}

/// Provides information about, and control over, the current [Node.js](https://nodejs.org) process.
@JS()
abstract class Process extends EventEmitter {

  /// The processor architecture that the process is currently running on.
  external String get arch;

  /// The command line arguments passed when the process was launched.
  ///
  /// The first element will be [execPath]. See [argv0] if access to the original value of `argv[0]` is needed.
  /// The second element will be the path to the JavaScript file being executed.
  /// The remaining elements will be any additional command line arguments.
  external List<String> get argv;

  /// The original value of `argv[0]` passed when Node.js starts.
  external String get argv0;

  /// If the process was spawned with an IPC channel, the [channel] property is a reference to the IPC channel.
  /// If no IPC channel exists, this property is `null`.
  external dynamic get channel;

  /// An object containing the JavaScript representation of the configure options used to compile the current Node.js executable.
  /// This is the same as the `config.gypi` file that was produced when running the `./configure` script.
  external dynamic get config;
  external set config(value);

  /// If the process is spawned with an IPC channel, the [connected] property will return `true` so long as the IPC channel is connected.
  /// It will return `false` after [disconnect] is called.
  external bool get connected;

  /// An object containing the user environment.
  external dynamic get env;

  /// The list of Node.js-specific command-line options passed when the process was launched.
  ///
  /// These options do not appear in the array returned by the [argv] property, and do not include the Node.js executable,
  /// the name of the script, or any options following the script name.
  /// These options are useful in order to spawn child processes with the same execution environment as the parent.
  external List<String> get execArgv;

  /// The absolute pathname of the executable that started the process.
  external String get execPath;

  /// The process exit code, when the process either exits gracefully, or is exited via [exit] without specifying a code.
  external int get exitCode;
  external set exitCode(int value);

  /// An alternative way of retrieving `require.main`.
  external Module get mainModule;

  /// The process identifier (PID) of the process.
  external int get pid;

  /// A string identifying the operating system platform on which the process is running.
  external String get platform;

  /// The PID of the current parent process.
  external int get ppid;

  /// The metadata related to the current release, including URLs for the source tarball and headers-only tarball.
  external Release get release;

  /// A stream connected to `stderr` (file descriptor `2`).
  /// It is a `Socket` (which is a [Duplex] stream) unless file descriptor `2` refers to a file, in which case it is a [Writable] stream.
  external Writable get stderr;

  /// A stream connected to `stdin` (file descriptor `0`).
  /// It is a `Socket` (which is a [Duplex] stream) unless file descriptor `0` refers to a file, in which case it is a [Readable] stream.
  external Readable get stdin;

  /// A stream connected to `stdout` (file descriptor `1`).
  /// It is a `Socket` (which is a [Duplex] stream) unless file descriptor `1` refers to a file, in which case it is a [Writable] stream.
  external Writable get stdout;

  /// The current process title (i.e. the value of the `ps` command).
  external String get title;
  external set title(String value);

  /// The Node.js version string.
  external String get version;

  /// An object listing the version strings of Node.js and its dependencies.
  external dynamic get versions;

  /// Causes the process to exit immediately and generate a core file.
  external void abort();

  /// Changes the current working [directory] of the process.
  external void chdir(String directory);

  /// Returns the user and system CPU time usage of the current process.
  /// The result of a previous call to [cpuUsage] can be passed as the argument to the function, to get a diff reading.
  external CpuUsage cpuUsage([CpuUsage previousValue]);

  /// Returns the current working directory of the process.
  external String cwd();

  /// If the process is spawned with an IPC channel, the method will close the IPC channel to the parent process,
  /// allowing the child process to exit gracefully once there are no other connections keeping it alive.
  ///
  /// The effect of calling [disconnect] is that same as calling the parent process's `ChildProcess.disconnect()`.
  external void disconnect();

  /// Allows to dynamically load shared objects.
  ///
  /// It is primarily used by [require] to load C++ Addons, and should not be used directly, except in special cases.
  /// In other words, [require] should be preferred over [dlopen], unless there are specific reasons.
  ///
  /// The [flags] argument allows to specify [dlopen] behavior. See the `OSDlopenConstants` documentation for details.
  ///
  /// If there are specific reasons to use [dlopen], it's often useful to use `require.resolve()` to look up the module's path.
  ///
  /// Note: An important drawback when calling [dlopen] is that the `module` instance must be passed.
  /// Functions exported by the C++ Addon will be accessible via `module.exports`.
  external void dlopen(Module module, String filename, [int flags]);

  /// Emits custom or application specific process warnings.
  /// These can be listened for by adding a handler to the `process.on("warning")` event.
  ///
  /// If [warning] is passed as a [JSError] object, the optional arguments are ignored.
  external void emitWarning(warning, [typeOrOptions, codeOrCtor, Function ctor]);

  /// Instructs Node.js to terminate the process synchronously with an exit status of [code].
  /// If [code] is omitted, [exit] uses either the success code `0` or the value of [exitCode] if it has been set.
  void exit([int code]);

  /// Returns the numerical effective group identity of process (see [`getegid()`](http://manpages.ubuntu.com/manpages/xenial/man2/getegid.2.html)).
  external int getegid();

  /// Returns the numerical effective user identity of the process (see [`getegid()`](http://manpages.ubuntu.com/manpages/xenial/man2/geteuid.2.html)).
  external int geteuid();

  /// Returns the numeric group identity of the process (see [`getegid()`](http://manpages.ubuntu.com/manpages/xenial/man2/getgid.2.html)).
  external int getgid();

  /// Returns a list with the supplementary group identifiers (see [`getegid()`](http://manpages.ubuntu.com/manpages/xenial/man2/getgroups.2.html)).
  external List<int> getgroups();

  /// Returns the numeric user identity of the process (see [`getegid()`](http://manpages.ubuntu.com/manpages/xenial/man2/getuid.2.html)).
  external int getuid();

  /// Returns a value indicating whether a callback has been set using [setUncaughtExceptionCaptureCallback].
  external bool hasUncaughtExceptionCaptureCallback();

  /// Returns the current high-resolution real time in a `[seconds, nanoseconds]` tuple,
  /// where `nanoseconds` is the remaining part of the real time that can't be represented in second precision.
  ///
  /// [time] is an optional parameter that must be the result of a previous [hrtime] call to diff with the current time.
  external List<int> hrtime([List<int> time]);

  /// Reads the `/etc/group` file and initializes the group access list, using all groups of which the [user] is a member.
  /// This is a privileged operation that requires that the process either have `root` access or the `CAP_SETGID` capability.
  ///
  /// [user] can be a user name or a numeric user identifier.
  /// [extraGroup] can be a group name or a numeric group identifier.
  external void initgroups(Object user, [Object extraGroup]);

  /// Sends the [signal] to the process identified by [pid].
  ///
  /// Even though the name of this method is `process.kill()`, it is really just a signal sender,
  /// like the [`kill`](http://manpages.ubuntu.com/manpages/xenial/man2/kill.2.html) system call.
  /// The signal sent may do something other than kill the target process.
  external void kill(int pid, [signal = 'SIGTERM']);

  /// Returns an object describing the memory usage of the process measured in bytes.
  external MemoryUsage memoryUsage();

  /// Adds the [callback] to the "next tick queue".
  /// Once the current turn of the event loop turn runs to completion, all callbacks currently in the next tick queue will be called.
  external void nextTick(Function callback, [Object arg1, Object arg2, Object arg3, Object arg4, Object arg5, Object arg6, Object arg7]);

  /// If Node.js is spawned with an IPC channel, the [send] method can be used to send messages to the parent process.
  /// Messages will be received as a `message` event on the parent's `ChildProcess` object.
  ///
  /// If Node.js was not spawned with an IPC channel, the [send] method will be not be available.
  ///
  /// The message goes through JSON serialization and parsing. The resulting message might not be the same as what is originally sent.
  external bool send(message, [sendHandleOrOptions, optionsOrCallback, CompletionCallback0 callback]);

  /// Sets the effective group identity of the process (see [`setegid()`](http://manpages.ubuntu.com/manpages/xenial/man2/setegid.2.html)).
  ///
  /// The [id] can be passed as either a numeric identifier or a group name string.
  /// If a group name is specified, the method blocks while resolving the associated numeric identifier.
  external void setegid(Object id);

  /// Sets the effective user identity of the process (see [`seteuid()`](http://manpages.ubuntu.com/manpages/xenial/man2/seteuid.2.html)).
  ///
  /// The [id] can be passed as either a numeric identifier or a username string.
  /// If a username is specified, the method blocks while resolving the associated numeric identifier.
  external void seteuid(Object id);

  /// Sets the group identity of the process (see [`setgid()`](http://manpages.ubuntu.com/manpages/xenial/man2/setgid.2.html)).
  ///
  /// The [id] can be passed as either a numeric identifier or a group name string.
  /// If a group name is specified, the method blocks while resolving the associated numeric identifier.
  external void setgid(Object id);

  /// Sets the supplementary group identifiers for the process (see [`setgroups()`](http://manpages.ubuntu.com/manpages/xenial/man2/setgroups.2.html)).
  /// This is a privileged operation that requires the process to have `root` or the `CAP_SETGID` capability.
  ///
  /// The [groups] list can contain numeric group identifiers, group names or both.
  external void setgroups(List groups);

  /// Sets the user identity of the process (see [`setuid()`](http://manpages.ubuntu.com/manpages/xenial/man2/setuid.2.html)).
  ///
  /// The [id] can be passed as either a numeric identifier or a username string.
  /// If a username is specified, the method blocks while resolving the associated numeric identifier.
  external void setuid(Object id);

  /// Sets a function that will be invoked when an uncaught exception occurs, which will receive the exception value itself as its first argument.
  external void setUncaughtExceptionCaptureCallback(void fn(JSError error));

  /// Sets or returns the process's file mode creation mask.
  ///
  /// Invoked without an argument, the current mask is returned,
  /// otherwise the `umask` is set to the argument value and the previous mask is returned.
  external int umask([int mask]);

  /// Returns the number of seconds the current process has been running.
  /// The returned value includes fractions of a second. Use [double.floor] to get whole seconds.
  external double uptime();
}

/// Provides information about a [Node.js](https://nodejs.org) release.
@JS()
abstract class Release {

  /// An absolute URL pointing to a `.tar.gz` file containing the source code of this release.
  external String get headersUrl;

  /// An absolute URL pointing to a `node.lib` file matching the architecture and version of this release.
  external String get libUrl;

  /// A string label identifying the [LTS](https://github.com/nodejs/Release) label for this release.
  /// This property only exists for LTS releases.
  external String get lts;

  /// A value that will always be `"node"` for Node.js.
  external String get name;

  /// An absolute URL pointing to a `.tar.gz` file containing only the source header files for this release.
  external String get sourceUrl;
}
